package com.fjhx.mes.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.production.dto.ProduceOrderDetailSelectDto;
import com.fjhx.mes.entity.production.po.ProductionOrder;
import com.fjhx.mes.entity.production.po.ProductionOrderDetail;
import com.fjhx.mes.entity.production.vo.ProductionOrderDetailVo;
import com.fjhx.mes.entity.production.vo.ProductionProcessesVo;
import com.fjhx.mes.mapper.production.ProduceOrderDetailMapper;
import com.fjhx.mes.mapper.production.ProductionProcessesMapper;
import com.fjhx.mes.service.production.ProduceOrderDetailService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产订单明细 服务实现类
 * </p>
 *
 * @author
 * @since 2024-01-10
 */
@Service
public class ProduceOrderDetailServiceImpl extends ServiceImpl<ProduceOrderDetailMapper, ProductionOrderDetail> implements ProduceOrderDetailService {

    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private ProductionProcessesMapper productionProcessesMapper;

    @Override
    public Page<ProductionOrderDetailVo> getPage(ProduceOrderDetailSelectDto dto) {
        IWrapper<ProductionOrderDetail> wrapper = getWrapper();

        //生产状态
        wrapper.eq("pod", ProductionOrderDetail::getProduceStatus, dto.getProduceStatus());
        //交期-时间范围过滤
        wrapper.ge("po", ProductionOrder::getDeliveryPeriod, dto.getStaDeliveryPeriod());
        wrapper.le("po", ProductionOrder::getDeliveryPeriod, dto.getEndDeliveryPeriod());
        //下单时间（创建时间）-时间范围过滤
        wrapper.ge("po", ProductionOrder::getCreateTime, dto.getBeginTime());
        wrapper.le("po", ProductionOrder::getCreateTime, dto.getEndTime());
        //关键字
        wrapper.keyword(dto.getKeyword(), new SqlField("po", ProductionOrder::getCode));

        wrapper.orderByDesc("po", ProductionOrder::getCreateTime);
        wrapper.orderByDesc("pod", ProductionOrderDetail::getId);

        Page<ProductionOrderDetailVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionOrderDetailVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //赋值产品信息
        productInfoService.attributeAssign(records, ProductionOrderDetail::getProductId, (item, product) -> {
            item.setProductCode(product.getCustomCode());
            item.setProductName(product.getName());
            item.setProductUnit(product.getUnit());
            item.setProductSpec(product.getSpec());
            item.setProductLength(product.getLength());
            item.setProductWidth(product.getWidth());
            item.setProductHeight(product.getHeight());
        });

        //赋值生产用时
        for (ProductionOrderDetailVo record : records) {
            record.setUsageTime(0L);
            Date produceTime = record.getProduceTime();
            Date finishTime = record.getFinishTime();
            if (ObjectUtil.isNotEmpty(produceTime) && ObjectUtil.isNotEmpty(finishTime)) {
                long usageTime = finishTime.getTime() - produceTime.getTime();
                record.setUsageTime(usageTime / 1000 / 60 / 60);
            }
        }

        //赋值工序列表
        List<Long> pIds = records.stream().map(ProductionOrderDetail::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(pIds)) {
            List<ProductionProcessesVo> processes = productionProcessesMapper.getProcessesByProductId(IWrapper.getWrapper().in("pi.id", pIds));
            Map<Long, List<ProductionProcessesVo>> processesMap = processes.stream().collect(Collectors.groupingBy(ProductionProcessesVo::getProductId));
            for (ProductionOrderDetailVo record : records) {
                List<ProductionProcessesVo> productionProcessesVo = processesMap.get(record.getProductId());
                record.setProductionProcessesList(productionProcessesVo);
            }
        }

        return page;
    }

}
