package com.fjhx.wms.entity.stock.emums;

import com.fjhx.common.enums.InOutType;
import com.ruoyi.common.exception.ServiceException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.HashMap;
import java.util.Map;

@AllArgsConstructor
@Getter
public enum StockWaitType {

    BORDER_OF_LINE_IN(InOutType.IN, JournalType.BORDER_OF_LINE_IN, 1, "线边回仓", ""),
    COMPLETION_IN(InOutType.IN, JournalType.COMPLETION_IN, 2, "完工入库", ""),
    PURCHASE_ARRIVAL_IN(InOutType.IN, JournalType.PURCHASE_ARRIVAL_IN, 3, "采购到货", ""),
    BACK_OUT(InOutType.OUT, JournalType.BACK_OUT, 4, "退货出库", ""),
    SALE_ORDER_OUT(InOutType.OUT, JournalType.SALES_OUT, 6, "销售订单出库", "order_info"),
    PRODUCTION_TASK_OUT(InOutType.OUT, JournalType.PRODUCTION_TASK_OUT, 7, "生产任务待出库", "production_task");


    private final static Map<Integer, StockWaitType> map = new HashMap<>();

    static {
        for (StockWaitType value : StockWaitType.values()) {
            map.put(value.detailType, value);
        }
    }

    /**
     * 出入库类型
     */
    private final InOutType type;
    /**
     * 出入库流水类型
     */
    private final JournalType journalType;
    /**
     * 详细类型
     */
    private final Integer detailType;

    /**
     * 信息类型说明
     */
    private final String detailTypeRemark;

    /**
     * 关联业务表名
     */
    private final String linkBusinessTableName;


    public static StockWaitType getEnum(Integer key) {
        return map.computeIfAbsent(key, item -> {
            throw new ServiceException("未知待出/入库类型");
        });
    }

}
